@@ -24,8 +24,6 @@ class FileCache:
         self.root_path = root_path or os.getcwd()
         self.current_memory_usage = 0
         self.file_futures = {}
-        self.file_sizes = {}
-        self.file_write_futures = {}
         self.file_access_times = []
         self.file_futures_lock = Lock()
         self.executor = ThreadPoolExecutor()
@@ -109,17 +107,12 @@ class FileCache:
         None
         """
         with self.file_futures_lock:
-            if file_name in self.file_futures:
-                if file_name in self.file_sizes:
-                    self.current_memory_usage -= self.file_sizes[file_name]
-                self.file_sizes[file_name] = memory_usage
+            # if not present, file has been removed while it was writing
+            info = self.file_futures.get(file_name)
+            if info is not None:
                 self.current_memory_usage += memory_usage
                 self.update_file_access_time(file_name)
-            else:
-                # file has been removed while it was writing
-                assert file_name not in self.file_sizes
-            if file_name in self.file_write_futures:
-                self.file_write_futures.pop(file_name)
+                self.file_futures[file_name] = (False, *info[1:])
 
     def update_file(self, file_name, new_file_contents, use_fsync=False):
         """
@@ -141,18 +134,19 @@ class FileCache:
         if claim > self.max_memory:
             raise MemoryError(f"requested file update larger than max_memory: {file_name} {claim} {self.max_memory}")
         with self.file_futures_lock:
-            write_future = self.file_write_futures.get(file_name)
-            if write_future is None:
-                # there's a race condition here where the file future could be evicted before the write completes
+            info = self.file_futures.get(file_name)
+            if info is None or not info[0]:
+                self._unload_file(file_name)
                 if not self.recover_memory(claim):
                     raise MemoryError(f"unable to recover memory for requsted file: {file_name} {claim}")
-                write_future = self.executor.submit(self._write_file, file_name, new_file_contents, use_fsync)
-                self.file_write_futures[file_name] = write_future
-                self.file_futures[file_name] = write_future
+                future = self.executor.submit(self._write_file, file_name, new_file_contents, use_fsync)
+                self.file_futures[file_name] = (True, claim, future)
                 write_applied = True
             else:
+                assert info[0]
+                future = info[-1]
                 write_applied = False
-        write_future.result()
+        future.result()
         return write_applied
 
     def _unload_file(self, file_name):
@@ -166,9 +160,10 @@ class FileCache:
         None
         """
         assert self.file_futures_lock.locked()
-        self.current_memory_usage -= self.file_sizes[file_name]
-        del self.file_futures[file_name]
-        del self.file_sizes[file_name]
+        info = self.file_futures.get(file_name)
+        if info is not None:
+            self.current_memory_usage -= info[1]
+            del self.file_futures[file_name]
 
     def unload_file(self, file_name):
         """
@@ -218,17 +213,14 @@ class FileCache:
         claim = os.path.getsize(full_file_path)
         if claim > self.max_memory:
             raise MemoryError(f"requested file larger than max_memory: {file_name} {claim} {self.max_memory}")
-        future = None
         with self.file_futures_lock:
-            future = self.file_futures.get(file_name)
-            if future is None:
+            info = self.file_futures.get(file_name)
+            if info is None:
                 if not self.recover_memory(claim):
                     raise MemoryError(f"unable to recover memory for requsted file: {file_name} {claim}")
                 future = self.executor.submit(self._load_file, file_name)
-                self.file_futures[file_name] = future
+                self.file_futures[file_name] = (False, claim, future)
             else:
-                write_future = self.file_write_futures.get(file_name)
-                if write_future is not None:
-                    write_future.result()
+                future = info[-1]
                 self.update_file_access_time(file_name)
         return future.result()
